From b9b9bf31c485246ec4f875d2398450197621fdd9 Mon Sep 17 00:00:00 2001
From: Jarett Stevens <jarett.stevens@gmail.com>
Date: Mon, 29 Jul 2013 23:35:01 -0500
Subject: [PATCH] vlock - added option to enable execution of arbitrary command
 (cmatrix, top -s, etc) during session locking.

---
 man/man1/vlock.1  |  5 +++++
 src/vlock/auth.c  | 39 +++++++++++++++++++++++++++++++++++++++
 src/vlock/parse.c | 12 +++++++++++-
 src/vlock/vlock.h |  6 ++++++
 4 files changed, 61 insertions(+), 1 deletion(-)

diff --git a/man/man1/vlock.1 b/man/man1/vlock.1
index 452837a..ad62ecf 100644
--- a/man/man1/vlock.1
+++ b/man/man1/vlock.1
@@ -37,6 +37,11 @@ Lock all console sessions and disable VC switching.
 .IP
 Lock the current session (this is the default).
 .PP
+.B -e <cmd>,--exec <cmd>
+.IP
+Execute <cmd> while locked.  The string "%sec" in <cmd> will be replaced 
+with the number of seconds the session has been locked.
+.PP
 .B -h,--help
 .IP
 Print a brief help message.
diff --git a/src/vlock/auth.c b/src/vlock/auth.c
index 71c8f15..2ca1d1d 100644
--- a/src/vlock/auth.c
+++ b/src/vlock/auth.c
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <syslog.h>
+#include <sys/time.h>
 
 #include "pam_auth.h"
 #include "vlock.h"
@@ -36,16 +37,54 @@
 /* Unrecognized PAM error timeout. */
 #define	ERROR_TIMEOUT	10
 
+extern int pam_misc_conv_die_time ;
+
 int
 get_password (pam_handle_t * pamh, const char *username, const char *tty)
 {
 	uid_t   uid = getuid ();
+	struct timeval time_lock ;
+	struct timeval time_now ;
+	char *sec_p ;
+
+	sec_p = strstr(o_lock_cmd, "%sec") ;
+
+	if (sec_p) {
+		*sec_p = 0 ; /* % becomes null end of first half */
+	}
+
+	/* time locked */
+	(void) gettimeofday(&time_lock, NULL);
 
 	for (;;)
 	{
 		int     rc;
+		char	sec_str[16];
+		char	cmdtmp[EXEC_CMD_LEN];
 		const char *msg;
 
+		/* if a command to execute is given */
+		if (o_lock_exec) {
+
+			/* if the command has %sec embedded for substitution */
+			if (sec_p) {
+				(void) gettimeofday(&time_now, NULL);
+				(void) sprintf(sec_str, "%u", (uint) (time_now.tv_sec - time_lock.tv_sec)) ;
+
+				/* splice the number of seconds into the command */
+				strncpy(cmdtmp, o_lock_cmd, EXEC_CMD_LEN) ;
+				strncat(cmdtmp, sec_str, EXEC_CMD_LEN - strlen(cmdtmp)) ;
+				strncat(cmdtmp, (sec_p + 4), EXEC_CMD_LEN - strlen(cmdtmp)) ;
+				system(cmdtmp) ;
+
+			} else { 
+				system(o_lock_cmd) ;
+			}
+
+			(void) gettimeofday(&time_now, NULL);
+			pam_misc_conv_die_time = time_now.tv_sec + ERROR_TIMEOUT ;
+		}
+
 		if (!pamh)
 		{
 			pamh = init_pam (username, tty, 1);
diff --git a/src/vlock/parse.c b/src/vlock/parse.c
index b7e5ef6..0044680 100644
--- a/src/vlock/parse.c
+++ b/src/vlock/parse.c
@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <getopt.h>
+#include <string.h>
 
 #include "vlock.h"
 #include "config.h"
@@ -36,6 +37,8 @@
  * 0 means current, 1 means all.
  */
 int     o_lock_all;
+int     o_lock_exec = 0 ;
+char    o_lock_cmd[EXEC_CMD_LEN] ;
 
 const char *
 locked_name (void)
@@ -62,6 +65,8 @@ show_help(void)
 	       "       switch to other virtual consoles.\n"
 	       "-a or --all: lock all virtual consoles by preventing other users\n"
 	       "       from switching virtual consoles.\n"
+	       "-e or --exec: execute command on locked terminal (ex: cmatrix). The\n"
+	       "       string %%sec will be replaced with the number of seconds locked.\n"
 	       "-v or --version: Print the version number of vlock and exit.\n"
 	       "-h or --help: Print this help message and exit.\n"),
 	       program_invocation_short_name, program_invocation_short_name);
@@ -74,13 +79,14 @@ parse (int ac, char *const av[])
 	static struct option long_options[] = {
 		{"current", 0, 0, 'c'},
 		{"all", 0, 0, 'a'},
+		{"exec", 1, 0, 'e'},
 		{"version", 0, 0, 'v'},
 		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0},
 	};
 	int     c;
 
-	while ((c = getopt_long(ac, av, "acvh", long_options, 0)) != -1)
+	while ((c = getopt_long(ac, av, "ace:vh", long_options, 0)) != -1)
 	{
 		switch (c)
 		{
@@ -90,6 +96,10 @@ parse (int ac, char *const av[])
 			case 'a':
 				o_lock_all = 1;
 				break;
+			case 'e':
+				o_lock_exec = 1;
+				strncpy(o_lock_cmd, optarg, EXEC_CMD_LEN) ;
+				break;
 			case 'v':
 				fprintf(stderr, "%s\n", VERSION);
 				exit(EXIT_SUCCESS);
diff --git a/src/vlock/vlock.h b/src/vlock/vlock.h
index eaeed0a..bac6e0c 100644
--- a/src/vlock/vlock.h
+++ b/src/vlock/vlock.h
@@ -42,3 +42,9 @@ extern int o_lock_all;
 
 /* Copy of the VT mode when the program was started. */
 extern int is_vt;
+
+/* optional command to execute during lock */
+extern int o_lock_exec ;
+extern char o_lock_cmd[] ;
+
+#define EXEC_CMD_LEN 128
-- 
1.8.3.4
